/* Import data */

proc import out=nishimura
   datafile='C:\chapter10\nishimura RF.csv'
   dbms=csv
	replace; 
   getnames=yes;
run;

/* select only studies that have evaluated RF */
data nishimura_RF;
	set nishimura;
	where test='RF';
run;

proc print;
run;


data nishimura_RF; 
	set  nishimura_RF;
	
	/* Create separate records for the diseased and non-diseased groups in each study
	The variable dis is the disease indicator which takes the value 0.5 if diseased 
	and -0.5 if not diseased. */
  	dis=0.5; pos=tp; n=tp+fn; output;
  	dis=-0.5; pos=fp; n=tn+fp; output;
run;

/* Ensure that both records for a study are clustered together  */
proc sort data=nishimura_RF;
  	by study_id dis;
run;

/* Run the Rutter and Gatsonis HSROC model with no covariates.
   request covariance matrices for model parameters ("cov") and
   also for additional esimates that are computed ("ecov") */

proc nlmixed data=nishimura_RF ecov cov ;

   /* set starting values for all model parameters to be estimated */

	parms alpha=2 theta=0 beta=0 s2ua=0 s2ut=0 ;

  	logitp = (theta + ut + (alpha + ua)*dis) * exp(-(beta)*dis);

  	p = exp(logitp)/(1+exp(logitp));

  	model pos ~ binomial(n,p);

	/* the random effects for accuracy (ua) and threshold (ut) are assumed to be
	   approximately normally distributed, both with mean zero and with variances
	   s2ua and s2ut respectively. The covariance of the random effects is set to 0. */

  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;

run;

	/* Create two dummy variables for to allow for the three RF measurement methods.
       LA is the referent method 
       Delete the 2 studies that did not report the method, and the study that used
       a different method.  */

data nishimura_RF; 
	set  nishimura_RF;
	if method_of_measurement ne "ELISA" and method_of_measurement ne "Nephelometry" and
	   method_of_measurement ne "LA" then delete;
  	rfm1=0;  rfm2=0; ;	
  	if method_of_measurement ="ELISA"  then rfm1=1;  			
  	if method_of_measurement ="Nephelometry"  then rfm2=1;
run;

/* Ensure that both records for a study are clustered together */
proc sort data=nishimura_RF;
  	by study_id dis;
run;

/* include covariates to allow accuracy, threshold and shape to vary by method */

proc nlmixed data=nishimura_RF ecov cov ;

	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0 b1=0 b2=0 ;

  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua + a1*rfm1 + a2*rfm2)*dis)*
            exp(-(beta + b1*rfm1 + b2*rfm2)*dis);

  	p = exp(logitp)/(1+exp(logitp));

  	model pos ~ binomial(n,p);

  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;

	/* parameter estimates for the methods of RF measurement; */
  	estimate 'alpha ELISA'  alpha + a1;  
	estimate 'theta ELISA'  theta + t1;
  	estimate 'beta ELISA' 	beta + b1;
	estimate 'alpha Nephelometry' alpha + a2;
	estimate 'theta Nephelometry' theta + t2;
	estimate 'beta Nephelometry' beta + b2;
run;


/* simplify the model to assume that all three curves have the same shape */
proc nlmixed data=nishimura_RF ecov cov ;

	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 a1=0 a2=0 t1=0 t2=0 ;

  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua + a1*rfm1 + a2*rfm2)*dis)*
            exp(-(beta)*dis);

  	p = exp(logitp)/(1+exp(logitp));

  	model pos ~ binomial(n,p);

  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;

	/* parameter estimates for the methods of RF measurement; */
  	estimate 'alpha ELISA'  alpha + a1;  
	estimate 'theta ELISA'  theta + t1;
	estimate 'alpha Nephelometry' alpha + a2;
	estimate 'theta Nephelometry' theta + t2;
run;

/* check assumption of normality for random effects */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;

run;


/* this model assumes that all three curves have the same shape and position.
   The position is the same because there are no covariates included for accuracy.
   Comparison with the previous model allows us to test whether accuracy varies by method. */

proc nlmixed data=nishimura_RF ecov cov ;

	parms alpha=2 theta=0 beta=0 s2ua=1 s2ut=1 t1=0 t2=0 ;

  	logitp = (theta + ut + t1*rfm1 + t2*rfm2 + (alpha + ua)*dis)*
            exp(-(beta)*dis);

  	p = exp(logitp)/(1+exp(logitp));

  	model pos ~ binomial(n,p);

  	random ut ua ~ normal([0,0],[s2ut,0,s2ua]) subject=study_id out=randeffs;
run;


/* check assumption of normality for random effects */
proc univariate data=randeffs plot normal;
  class effect;
  var estimate;

run;

